/**
 * Created by abernh on 2016/02/04
 * Updated
 *
 * v5   2017/11/09  upgraded to save also session IDs in cookie
 * v4   2017/07/04  ie8 fix - replaced cookie.split.map by "for" loop
 * v3   2017/06/21  polyfill for Date.now
 * v2   2017/06/21  removed console log due to crashes in ie8
 */

(function(){
    /* polyfill Date.now - ie8 */
    if (!Date.now) {
        Date.now = function now() {
            return new Date().getTime();
        };
    }

    const SESSION_ID = (function(){
        var playsidMatch = location.href.match(/PLAYSID=([a-z0-9]+)/),
            res;

        if(playsidMatch) {
            res = playsidMatch[1];

        }else{
            console.error('PLAYSID not found in current URI');
            res = "noplaysid" + (Math.random() * 9999).toFixed();
        }

        return res;
    })() // retrieved from location.href

    /**
     * simple object that allows to read and set the status of the current survey cookie.
     * used to weed-out panelists which already took part in the survey once based on cookies.
     *
     * @param surveyId
     * @returns {SurveyCookie}
     * @constructor
     */
    var
        SurveyStati = {
            UNSET: "UNSET",
            STARTED: "STARTED",
            COMPLETED: "COMPLETED",
            SCREENEDOUT: "SCREENEDOUT"
        },
        SurveyCookie = function(surveyId){
            var vars = {
                    surveyId: surveyId
                },
                fn = {
                    get: function(){
                        var cookies = {},
                            cookieArr = document.cookie.split("; "),
                            tmpVal;

                        for(var i in cookieArr){
                            tmpVal = cookieArr[i];
                            if(!tmpVal.match(/.+=.+/)) continue;

                            var tmp = tmpVal.toString().split("=");

                            try{
                                tmp[1] = decodeURIComponent(tmp[1]);
                                tmp[1] = JSON.parse(tmp[1]);
                            }catch(err){
                                try{
                                    tmp[1] = JSON.parse(atob(tmp[1]));
                                }catch(err2){

                                }
                            }

                            cookies[tmp[0]] = tmp[1];
                        }
                        //console.log('cookies', cookies, cookies[vars.surveyId]);

                        var cookieDataRaw = cookies[vars.surveyId],
                            cookieData;

                        // compatibility to previous version which only saved the cookie state
                        if(!cookieDataRaw){
                            cookieData = {
                                status: SurveyStati.UNSET,
                                sessions: {}
                            }
                        }else if(typeof cookieDataRaw === "string"){
                            cookieData = {
                                status: cookieDataRaw,
                                sessions: {}
                            }
                        }else if(typeof cookieDataRaw === "object" && cookieDataRaw.status !== undefined){
                            cookieData = cookieDataRaw

                        }else{
                            console.error("bad cookie data", cookieDataRaw);

                        }

                        return cookieData;
                    },
                    getSessions: function(){
                        var state = this.get(),
                            sessions = [];

                        for(var sid in state.sessions){
                            var lastSessionState = state.sessions[sid][state.sessions[sid].length - 1];
                            sessions.push([
                                sid,
                                lastSessionState.timestamp,
                                lastSessionState.status
                            ].join(";"))
                        }

                        return sessions;
                    },
                    set: function(status){
                        var maxage_ms = 365*24*60*60*1000;

                        var cookieData = this.get(),
                            cookieDataRaw;

                        cookieData.status = status;
                        cookieData.sessions[SESSION_ID] = cookieData.sessions[SESSION_ID] || [];
                        cookieData.sessions[SESSION_ID].push({
                            timestamp: (Date.now() / 1000).toFixed(),
                            date: (new Date()).toUTCString(),
                            status: status
                        });

                        cookieDataRaw = btoa(JSON.stringify(cookieData));

                        document.cookie = vars.surveyId + "=" + cookieDataRaw + ";max-age=" + maxage_ms + ";expires" + (new Date(Date.now() + maxage_ms)).toUTCString();
                    },
                    is: function(/*..stati*/){
                        var is = true;
                        for(var i in arguments){ is = is && (fn.get().status === arguments[i]); }
                        return is;
                    }
                };
            for(var i in fn) this[i] = fn[i];
            return this;
        };

    SurveyCookie._STATUS = SurveyStati;
    window.SurveyCookie = SurveyCookie;
}());
